//
//  MOIconDownloaderManager.h
//  MOThumbnailTableView
//
//  Created by Jan Christiansen on 10/1/12.
//  Copyright (c) 2012 monoid - Software Development and Consulting - Jan Christiansen.
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//  copyright notice, this list of conditions and the following
//  disclaimer in the documentation and/or other materials provided
//  with the distribution.
//
//  * Neither the name of Monoid - Development and Consulting -
//  Jan Christiansen nor the names of other
//  contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import <Foundation/Foundation.h>
#import "MOIconDownloader.h"


@class MOIconDownloaderManager;


/**
 * Handles several simultaneous icon downloads and only starts a new one
 * if none is currently running for a specific icon
 */
@interface MOIconDownloaderManager : NSObject<MOIconDownloaderDelegate>


/// @name Getting and Setting Properties

/// Delegate that is informed when a icon finished downloading.
@property(assign, nonatomic) id<MOIconDownloaderDelegate> delegate;

/// All icon names are relative to this basis URL
@property(strong, nonatomic) NSURL *iconBasisURL;


/// @name Initializing

/**
 * Initialize an icon downloader manager
 *
 * @param iconBasisURL all icon names are relative to this basis URL
 */
- (id)initWithIconBasisURL:(NSURL *)iconBasisURL;


/// @name Starting and Stopping downloads

/**
 * Start downloading an icon and pass the user info to the downloader
 *
 * @param iconFileName name of the icon relative to `iconBasisURL`
 * @param userInfo dictionary with user info that can be accessed from the
 *        downloader when download finishes
 */
- (void)startDownloadOfIcon:(NSString *)iconFileName
               withUserInfo:(NSDictionary *)userInfo;


/// Stop all downloads
- (void)cancelDownloads;


@end
