//
//  MOThumbnailTableViewCell.m
//  MOThumbnailTableView
//
//  Created by Jan Christiansen on 11/5/12.
//  Copyright (c) 2012 monoid - Software Development and Consulting - Jan Christiansen.
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//  copyright notice, this list of conditions and the following
//  disclaimer in the documentation and/or other materials provided
//  with the distribution.
//
//  * Neither the name of Monoid - Development and Consulting -
//  Jan Christiansen nor the names of other
//  contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "MOThumbnailTableViewCell.h"


static const CGFloat kDefaultLeftThumbnailMargin = 10;

static const CGFloat kDefaultRightThumbnailMargin = 10;

static const CGFloat kDefaultRightLabelMargin = 10;


@interface MOThumbnailTableViewCell ()


@property(strong, nonatomic) UIImageView *thumbnailView;

@property(strong, nonatomic) UILabel *thumbnailLabel;


@end


@implementation MOThumbnailTableViewCell


// font size of label is not initialized correctly for the first table view
// cell, thus, we initialize it manually
- (id)initWithStyle:(UITableViewCellStyle)style
    reuseIdentifier:(NSString *)reuseIdentifier {

    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        _thumbnailMaxWidth = 30;
        _thumbnailMaxHeight = 30;
        _thumbnailLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _thumbnailLabel.backgroundColor = [UIColor clearColor];
        [self.contentView addSubview:_thumbnailLabel];
        _thumbnailView = [[UIImageView alloc] initWithFrame:CGRectZero];
        [self.contentView addSubview:_thumbnailView];
    }
    return self;
}

- (void)layoutSubviews {
    
    [super layoutSubviews];

    self.thumbnailLabel.frame = CGRectMake(kDefaultLeftThumbnailMargin + _thumbnailMaxWidth + kDefaultRightThumbnailMargin,
                                           15,
                                           self.contentView.bounds.size.width - (self.thumbnailView.frame.origin.x + self.thumbnailMaxWidth + kDefaultRightThumbnailMargin + kDefaultRightLabelMargin),
                                           20);
}


#pragma mark - Getting and Setting Properties

- (NSString *)name {
    
    return self.thumbnailLabel.text;
}

- (void)setName:(NSString *)name {

    self.thumbnailLabel.text = name;
}

- (UIImage *)thumbnail {

    return self.thumbnailView.image;
}

- (void)setThumbnail:(UIImage *)thumbnail {

    self.thumbnailView.image = thumbnail;

    if (thumbnail) {
        CGFloat widthFactor = self.thumbnailMaxWidth / thumbnail.size.width;
        CGFloat heightFactor = self.thumbnailMaxHeight / thumbnail.size.height;
        CGFloat factor = MIN(widthFactor, heightFactor);
        CGRect newFrame = CGRectMake(10, 10, factor * thumbnail.size.width, factor * thumbnail.size.height);
        self.thumbnailView.frame = newFrame;
    }
}

- (void)setThumbnailMaxWidth:(CGFloat)thumbnailMaxWidth {

    _thumbnailMaxWidth = thumbnailMaxWidth;
    [self setNeedsLayout];
}


@end
