//
//  MOThumbnailViewController.m
//  MOThumbnailTablewView
//
//  Created by Jan Christiansen on 10/1/12.
//  Copyright (c) 2012 monoid - Software Development and Consulting - Jan Christiansen.
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//  copyright notice, this list of conditions and the following
//  disclaimer in the documentation and/or other materials provided
//  with the distribution.
//
//  * Neither the name of Monoid - Development and Consulting -
//  Jan Christiansen nor the names of other
//  contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "NSArray+FunctionalStyle.h"
#import "NSDictionary+PrimitiveTypes.h"

#import "MOThumbnailTableViewController.h"
#import "MOThumbnailTableViewCell.h"
#import "MOThumbnailGroupSpecification.h"
#import "MOThumbnailSpecification.h"
#import "MOIconDownloaderManager.h"


const CGFloat kDefaultThumbnailMaxWidth = 60;

const CGFloat kDefaultThumbnailMaxHeight = 30;

static NSString *kIndexPathDictionaryKey = @"indexPath";


@interface MOThumbnailTableViewController ()


@property(assign, nonatomic) BOOL respondsToPresentViewController;

@property(assign, nonatomic) BOOL respondsToDismissViewController;


//

@property(strong, nonatomic) MOThumbnailTableSpecification *specification;

// dictionary mapping indexpaths to UIImages
@property(strong, nonatomic) NSMutableDictionary *thumbnails;


@property(strong, nonatomic) MOIconDownloaderManager *iconDownloaderManager;


@end


@implementation MOThumbnailTableViewController


- (id)initWithCoder:(NSCoder *)aDecoder
  specificationFile:(NSString *)specificationFile {

    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initializeWithSpecificationFile:specificationFile];
    }
    return self;
}

- (id)initWithStyle:(UITableViewStyle)style
specificationFile:(NSString *)specificationFile {

    NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:specificationFile];
    MOThumbnailTableSpecification *specification = [[MOThumbnailTableSpecification alloc] initWithDictionary:dict];
    return [self initWithStyle:style specification:specification];
}

// array of group specifications
- (id)initWithStyle:(UITableViewStyle)style
specification:(MOThumbnailTableSpecification *)specification {

    self = [super initWithStyle:style];
    if (self) {

    }
    return self;
}

- (void)initializeWithSpecificationFile:(NSString *)specificationFile {

    NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:specificationFile];
    MOThumbnailTableSpecification *specification = [[MOThumbnailTableSpecification alloc] initWithDictionary:dict];
    [self initializeWithSpecification:specification];
}

- (void)initializeWithSpecification:(MOThumbnailTableSpecification *)specification {

    _respondsToPresentViewController = [self respondsToSelector:@selector(presentViewController:animated:completion:)];
    _respondsToDismissViewController = [self respondsToSelector:@selector(dismissViewControllerAnimated:completion:)];
    
    _thumbnailMaxWidth = kDefaultThumbnailMaxWidth;
    _thumbnailMaxHeight = kDefaultThumbnailMaxHeight;
    
    _localizedStrings = NO;
    
    _specification = specification;
    
    _thumbnails = @{}.mutableCopy;
    
    self.title = specification.title;
}

- (void)viewDidLoad {

    [super viewDidLoad];

    UIBarButtonItem *backButton = [[UIBarButtonItem alloc] initWithTitle:@"Back"
                                                                   style:UIBarButtonItemStyleDone
                                                                  target:self
                                                                  action:@selector(dismiss)];

    self.navigationItem.rightBarButtonItem = backButton;
}

- (void)viewWillAppear:(BOOL)animated {

    [super viewWillAppear:animated];

    [self.tableView reloadData];
}

- (void)dismiss {

    if (_respondsToDismissViewController) {
        [self dismissViewControllerAnimated:YES completion:nil];
    } else {
        [self dismissModalViewControllerAnimated:YES];
    }
}

- (void)didReceiveMemoryWarning {

    [super didReceiveMemoryWarning];
}


#pragma mark - Lazy Initialization

- (MOIconDownloaderManager *)iconDownloaderManager {

    if (!_iconDownloaderManager) {
        _iconDownloaderManager = [[MOIconDownloaderManager alloc] init];
        _iconDownloaderManager.delegate = self;
    }
    return _iconDownloaderManager;
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {

    return self.specification.groups.count;
}

- (NSInteger)tableView:(UITableView *)tableView
 numberOfRowsInSection:(NSInteger)section {

    MOThumbnailGroupSpecification *groupSpecification = [self.specification.groups objectAtIndex:section];
    return groupSpecification.thumbnails.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {

    MOThumbnailGroupSpecification *groupSpecification = [self.specification.groups objectAtIndex:indexPath.section];
    MOThumbnailSpecification *specification = [groupSpecification.thumbnails objectAtIndex:indexPath.row];

    static NSString *CellIdentifier = @"ThumbnailCell";
    MOThumbnailTableViewCell *thumbnailCell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (!thumbnailCell) {
        thumbnailCell = [[MOThumbnailTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                                        reuseIdentifier:CellIdentifier];
        thumbnailCell.thumbnailMaxWidth = self.thumbnailMaxWidth;
        thumbnailCell.thumbnailMaxHeight = self.thumbnailMaxHeight;
    }
    thumbnailCell.name = specification.name;

    UIImage *thumbnail = [self.thumbnails objectForKey:indexPath];
    // if we have already used this thumbnail we simply display it
    if (thumbnail) {
        thumbnailCell.thumbnail = thumbnail;
    } else {
        if (specification.imageURL.isFileURL) {
            // if it is a local file add it to the arry and set the thumbnail
            UIImage *image = [UIImage imageNamed:specification.imageURL.path];
            if (!image) {
                NSLog(@"Image with path %@ does not exist", specification.imageURL.path);
            }
            [self.thumbnails setObject:image forKey:indexPath];
            thumbnailCell.thumbnail = image;
        } else {
            // if it is not a local file we start a download of the file
            NSDictionary *userInfo = [NSDictionary dictionaryWithObject:indexPath
                                                                 forKey:kIndexPathDictionaryKey];
            [self.iconDownloaderManager startDownloadOfIcon:specification.imageURL.absoluteString
                                               withUserInfo:userInfo];
            thumbnailCell.thumbnail = nil;
        }
    }

    thumbnailCell.selectionStyle = UITableViewCellSelectionStyleNone;

    return thumbnailCell;
}

- (NSString *)tableView:(UITableView *)tableView
titleForHeaderInSection:(NSInteger)section {

    MOThumbnailGroupSpecification *groupSpecification = [self.specification.groups objectAtIndex:section];
    return groupSpecification.title;
}

- (CGFloat)tableView:(UITableView *)__unused tableView
heightForRowAtIndexPath:(NSIndexPath *)__unused indexPath {

    return 50;
}


#pragma mark - MOIconDownloaderDelegate Methods

- (void)iconDownloader:(MOIconDownloader *)iconDownloader
           didLoadIcon:(UIImage *)icon {

    NSIndexPath *indexPath = [iconDownloader.userInfo objectForKey:kIndexPathDictionaryKey];
    [self.thumbnails setObject:icon forKey:indexPath];
    [self.tableView reloadData];
}

- (void)iconDownloader:(MOIconDownloader *)iconDownloader
      didFailWithError:(NSError *)error {
    
    // we do nothing if an error occured while downloading an image for the
    // table view, that is, we simply do not display the image
}


@end
